@@ -1,8 +1,11 @@
 //! Module for handling content types
 
 use std::collections::BTreeMap;
+use std::convert::TryFrom;
+use std::fmt::{Display, Formatter};
 use std::str::{from_utf8, FromStr};
 
+use anyhow::anyhow;
 use itertools::Itertools;
 use lazy_static::*;
 use log::*;
@@ -267,6 +270,43 @@ pub fn detect_content_type_from_bytes(s: &[u8]) -> Option<ContentType> {
   }
 }
 
+/// Override of the content type
+#[derive(Debug, Clone, Copy, PartialOrd, PartialEq, Ord, Eq, Hash, Serialize, Deserialize)]
+pub enum ContentTypeHint {
+  BINARY,
+  TEXT,
+  DEFAULT
+}
+
+impl Default for ContentTypeHint {
+  fn default() -> Self {
+    ContentTypeHint::DEFAULT
+  }
+}
+
+impl Display for ContentTypeHint {
+  fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+    match self {
+      ContentTypeHint::BINARY => write!(f, "BINARY"),
+      ContentTypeHint::TEXT => write!(f, "TEXT"),
+      ContentTypeHint::DEFAULT => write!(f, "DEFAULT")
+    }
+  }
+}
+
+impl TryFrom<&str> for ContentTypeHint {
+  type Error = anyhow::Error;
+
+  fn try_from(value: &str) -> Result<Self, Self::Error> {
+    match value {
+      "BINARY" => Ok(ContentTypeHint::BINARY),
+      "TEXT" => Ok(ContentTypeHint::TEXT),
+      "DEFAULT" => Ok(ContentTypeHint::DEFAULT),
+      _ => Err(anyhow!("'{}' is not a valid value for ContentTypeHint", value))
+    }
+  }
+}
+
 #[cfg(test)]
 mod tests {
   use expectest::prelude::*;
