@@ -51,6 +51,7 @@ impl JsonToNum<u16> for u16 {
 pub fn json_to_string(value: &Value) -> String {
   match value {
     Value::String(s) => s.clone(),
+    Value::Null => String::default(),
     _ => value.to_string()
   }
 }
@@ -166,26 +167,64 @@ pub fn body_from_json(request: &Value, fieldname: &str, headers: &Option<HashMap
           });
           if content_type.is_json() {
             match serde_json::from_str::<JsonParsable>(&s) {
-              Ok(_) => OptionalBody::Present(s.clone().into(), Some(content_type)),
-              Err(_) => OptionalBody::Present(format!("\"{}\"", s).into(), Some(content_type))
+              Ok(_) => OptionalBody::Present(s.clone().into(), Some(content_type), None),
+              Err(_) => OptionalBody::Present(format!("\"{}\"", s).into(), Some(content_type), None)
             }
           } else if content_type.is_text() {
-            OptionalBody::Present(s.clone().into(), Some(content_type))
+            OptionalBody::Present(s.clone().into(), Some(content_type), None)
           } else {
             match decode(s) {
-              Ok(bytes) => OptionalBody::Present(bytes.into(), None),
-              Err(_) => OptionalBody::Present(s.clone().into(), None)
+              Ok(bytes) => OptionalBody::Present(bytes.into(), None, None),
+              Err(_) => OptionalBody::Present(s.clone().into(), None, None)
             }
           }
         }
       },
       Value::Null => OptionalBody::Null,
-      _ => OptionalBody::Present(v.to_string().into(), None)
+      _ => OptionalBody::Present(v.to_string().into(), None, None)
     },
     None => OptionalBody::Missing
   }
 }
 
+/// Deep merges the other value into the given value
+pub fn json_deep_merge(value: &Value, other: &Value) -> Value {
+  match (value, other) {
+    (Value::Array(items), Value::Array(other_items)) => {
+      let mut values = items.clone();
+      values.extend(other_items.iter().cloned());
+      Value::Array(values)
+    },
+    (Value::Object(entries), Value::Object(other_entries)) => {
+      let map = entries.iter()
+        .chain(other_entries.iter())
+        .fold(serde_json::Map::new(), |mut m, (k, v)| {
+          let value = if let Some(value) = m.get(k) {
+            json_deep_merge(value, v)
+          } else {
+            v.clone()
+          };
+          m.insert(k.clone(), value);
+          m
+      });
+      Value::Object(map)
+    },
+    _ => other.clone()
+  }
+}
+
+/// If the JSON value is empty
+pub fn is_empty(value: &Value) -> bool {
+  match value {
+    Value::Null => true,
+    Value::Bool(_) => false,
+    Value::Number(_) => false,
+    Value::String(s) => s.is_empty(),
+    Value::Array(a) => a.is_empty(),
+    Value::Object(o) => o.is_empty()
+  }
+}
+
 #[cfg(test)]
 mod tests {
   use expectest::expect;
@@ -214,7 +253,7 @@ mod tests {
   #[test]
   fn json_to_string_test() {
     expect!(json_to_string(&Value::from_str("\"test string\"").unwrap())).to(be_equal_to("test string".to_string()));
-    expect!(json_to_string(&Value::from_str("null").unwrap())).to(be_equal_to("null".to_string()));
+    expect!(json_to_string(&Value::from_str("null").unwrap())).to(be_equal_to("".to_string()));
     expect!(json_to_string(&Value::from_str("100").unwrap())).to(be_equal_to("100".to_string()));
     expect!(json_to_string(&Value::from_str("100.10").unwrap())).to(be_equal_to("100.1".to_string()));
     expect!(json_to_string(&Value::from_str("{}").unwrap())).to(be_equal_to("{}".to_string()));
@@ -248,7 +287,7 @@ mod tests {
      "#).unwrap();
     let headers = headers_from_json(&json);
     let body = body_from_json(&json, "body", &headers);
-    expect!(body).to(be_equal_to(OptionalBody::Present("\"This is a string\"".into(), Some("text/plain".into()))));
+    expect!(body).to(be_equal_to(OptionalBody::Present("\"This is a string\"".into(), Some("text/plain".into()), None)));
   }
 
   #[test]
@@ -263,7 +302,7 @@ mod tests {
      "#).unwrap();
     let headers = headers_from_json(&json);
     let body = body_from_json(&json, "body", &headers);
-    expect!(body).to(be_equal_to(OptionalBody::Present("\"This is a string\"".into(), Some("text/html".into()))));
+    expect!(body).to(be_equal_to(OptionalBody::Present("\"This is a string\"".into(), Some("text/html".into()), None)));
   }
 
   #[test]
@@ -278,7 +317,7 @@ mod tests {
      "#).unwrap();
     let headers = headers_from_json(&json);
     let body = body_from_json(&json, "body", &headers);
-    expect!(body).to(be_equal_to(OptionalBody::Present("\"This is actually a JSON string\"".into(), Some("application/json".into()))));
+    expect!(body).to(be_equal_to(OptionalBody::Present("\"This is actually a JSON string\"".into(), Some("application/json".into()), None)));
   }
 
   #[test]
@@ -293,7 +332,7 @@ mod tests {
      "#).unwrap();
     let headers = headers_from_json(&json);
     let body = body_from_json(&json, "body", &headers);
-    expect!(body).to(be_equal_to(OptionalBody::Present("\"This is actually a JSON string\"".into(), Some("application/json".into()))));
+    expect!(body).to(be_equal_to(OptionalBody::Present("\"This is actually a JSON string\"".into(), Some("application/json".into()), None)));
   }
 
   #[test]
@@ -308,7 +347,7 @@ mod tests {
      "#).unwrap();
     let headers = headers_from_json(&json);
     let body = body_from_json(&json, "body", &headers);
-    expect!(body).to(be_equal_to(OptionalBody::Present("{\"test\":true}".into(), Some("application/json".into()))));
+    expect!(body).to(be_equal_to(OptionalBody::Present("{\"test\":true}".into(), Some("application/json".into()), None)));
   }
 
   #[test]
@@ -354,7 +393,7 @@ mod tests {
       }
      "#).unwrap();
     let body = body_from_json(&json, "body", &None);
-    expect!(body).to(be_equal_to(OptionalBody::Present("{\"test\":true}".into(), None)));
+    expect!(body).to(be_equal_to(OptionalBody::Present("{\"test\":true}".into(), None, None)));
   }
 
   #[test]
@@ -382,6 +421,66 @@ mod tests {
       }
      "#).unwrap();
     let body = body_from_json(&json, "body", &None);
-    expect!(body).to(be_equal_to(OptionalBody::Present("<?xml version=\"1.0\"?> <body></body>".into(), Some("application/xml".into()))));
+    expect!(body).to(be_equal_to(OptionalBody::Present("<?xml version=\"1.0\"?> <body></body>".into(), Some("application/xml".into()), None)));
+  }
+
+  #[test]
+  fn deep_merge_with_primitives() {
+    let value = Value::Bool(true);
+    expect!(json_deep_merge(&value, &Value::Null)).to(be_equal_to(Value::Null));
+    expect!(json_deep_merge(&value, &Value::String("test".to_string()))).to(be_equal_to(Value::String("test".to_string())));
+  }
+
+  #[test]
+  fn deep_merge_with_empty_collections() {
+    let value = Value::Array(vec![]);
+    expect!(json_deep_merge(&value, &Value::Array(vec![]))).to(be_equal_to(Value::Array(vec![])));
+
+    let value = Value::Object(Default::default());
+    expect!(json_deep_merge(&value, &Value::Object(Default::default()))).to(be_equal_to(Value::Object(Default::default())));
+  }
+
+  #[test]
+  fn deep_merge_with_simple_objects() {
+    let value = json!({ "a": null });
+    expect!(json_deep_merge(&value, &json!({ "b": true }))).to(be_equal_to(json!({ "a": null, "b": true })));
+    expect!(json_deep_merge(&value, &json!({ "b": true, "a": false }))).to(be_equal_to(json!({ "a": false, "b": true })));
+  }
+
+  #[test]
+  fn deep_merge_with_simple_arrays() {
+    let value = json!([1, 2, 3]);
+    expect!(json_deep_merge(&value, &json!([4, 5]))).to(be_equal_to(json!([1, 2, 3, 4, 5])));
+  }
+
+
+  #[test]
+  fn deep_merge_with_collections_with_different_types() {
+    let value = json!({
+      "a": { "b": true },
+      "b": [ true ]
+    });
+    expect!(json_deep_merge(&value, &json!({ "a": true }))).to(be_equal_to(json!({ "a": true, "b": [ true ] })));
+    expect!(json_deep_merge(&value, &json!({ "b": true }))).to(be_equal_to(json!({ "a": { "b": true }, "b": true })));
+  }
+
+  #[test]
+  fn deep_merge_with_collections_recursively_merges_the_collections() {
+    let value = json!({
+      "a": { "b": true },
+      "b": [ true ]
+    });
+    let value2 = json!({
+      "a": { "b": false },
+      "b": [ false ]
+    });
+    expect!(json_deep_merge(&value, &value2)).to(be_equal_to(json!({
+      "a": { "b": false },
+      "b": [ true, false ]
+    })));
+    expect!(json_deep_merge(&value, &value)).to(be_equal_to(json!({
+      "a": { "b": true },
+      "b": [ true, true ]
+    })));
   }
 }
