@@ -1,15 +1,16 @@
 //! V4 specification Pact
 
 use std::cmp::Ordering;
-use std::collections::{BTreeMap, HashSet};
+use std::collections::{BTreeMap, HashSet, HashMap};
 use std::path::Path;
 use std::sync::{Arc, Mutex};
 
 use anyhow::{anyhow, Context};
 use itertools::EitherOrBoth::{Both, Left, Right};
 use itertools::Itertools;
+use log::{warn, trace};
 use maplit::btreemap;
-use serde_json::{json, Value};
+use serde_json::{json, Value, Map};
 
 use crate::{Consumer, PactSpecification, Provider};
 #[cfg(not(target_family = "wasm"))] use crate::file_utils::with_read_lock;
@@ -22,6 +23,7 @@ use crate::sync_pact::RequestResponsePact;
 use crate::v4::interaction::{interactions_from_json, V4Interaction};
 use crate::v4::V4InteractionType;
 use crate::verify_json::{json_type_of, PactFileVerificationResult, PactJsonVerifier, ResultLevel};
+use crate::plugins::PluginData;
 
 /// V4 spec Struct that represents a pact between the consumer and provider of a service.
 #[derive(Debug, Clone, PartialEq)]
@@ -33,7 +35,9 @@ pub struct V4Pact {
   /// List of messages between the consumer and provider.
   pub interactions: Vec<Box<dyn V4Interaction>>,
   /// Metadata associated with this pact.
-  pub metadata: BTreeMap<String, Value>
+  pub metadata: BTreeMap<String, Value>,
+  /// Plugin data associated with this pact
+  pub plugin_data: Vec<PluginData>
 }
 
 impl V4Pact {
@@ -50,7 +54,24 @@ impl V4Pact {
       .collect();
 
     md_map.insert("pactSpecification".to_string(), json!({"version" : PactSpecification::V4.version_str()}));
-    md_map.insert("pactRust".to_string(), json!({"version" : PACT_RUST_VERSION.unwrap_or("unknown")}));
+
+    let version_entry = md_map.entry("pactRust")
+      .or_insert(Value::Object(Map::default()));
+    if let Value::Object(map) = version_entry {
+      map.insert("models".to_string(), Value::String(PACT_RUST_VERSION.unwrap_or("unknown").to_string()));
+    }
+
+    if !self.plugin_data.is_empty() {
+      let mut v = vec![];
+      for plugin in &self.plugin_data {
+        match plugin.to_json() {
+          Ok(json) => v.push(json),
+          Err(err) => warn!("Could not convert plugin data to JSON - {}", err)
+        }
+      }
+      md_map.insert("plugins".to_string(), Value::Array(v));
+    }
+
     Value::Object(md_map)
   }
 
@@ -64,6 +85,48 @@ impl V4Pact {
     let interaction_types: HashSet<_> = self.interactions.iter().map(|i| i.v4_type()).collect();
     interaction_types.len() > 1
   }
+
+  /// Returns all the interactions of the given type
+  pub fn filter_interactions(&self, interaction_type: V4InteractionType) -> Vec<Box<dyn Interaction + Send + Sync>> {
+    self.interactions.iter()
+      .filter(|i| i.v4_type() == interaction_type)
+      .map(|i| i.boxed())
+      .collect()
+  }
+
+  fn add_plugin_data(&mut self, other_data: &PluginData) {
+    if let Some(data) = self.plugin_data.iter_mut()
+      .find(|data| data.name == other_data.name && data.version == other_data.version) {
+      data.merge(&other_data.configuration);
+    } else {
+      self.plugin_data.push(other_data.clone());
+    }
+  }
+
+  fn extract_plugin_data(metadata: &mut BTreeMap<String, Value>) -> Vec<PluginData> {
+    if let Some(plugin_data) = metadata.remove("plugins") {
+      match plugin_data {
+        Value::Array(items) => {
+          let mut v = vec![];
+
+          for item in &items {
+            match serde_json::from_value::<PluginData>(item.clone()) {
+              Ok(data) => v.push(data),
+              Err(err) => warn!("Could not convert '{}' into PluginData format - {}", item, err)
+            };
+          }
+
+          v
+        }
+        _ => {
+          warn!("'{}' is not valid plugin data", plugin_data);
+          vec![]
+        }
+      }
+    } else {
+      vec![]
+    }
+  }
 }
 
 impl Pact for V4Pact {
@@ -75,8 +138,8 @@ impl Pact for V4Pact {
     self.provider.clone()
   }
 
-  fn interactions(&self) -> Vec<&dyn Interaction> {
-    self.interactions.iter().map(|i| i.to_super()).collect()
+  fn interactions(&self) -> Vec<Box<dyn Interaction + Send + Sync>> {
+    self.interactions.iter().map(|i| i.boxed()).collect()
   }
 
   fn metadata(&self) -> BTreeMap<String, BTreeMap<String, String>> {
@@ -96,7 +159,9 @@ impl Pact for V4Pact {
       PactSpecification::V4 => Ok(json!({
         "consumer": self.consumer.to_json(),
         "provider": self.provider.to_json(),
-        "interactions": Value::Array(self.interactions.iter().map(|i| i.to_json()).collect()),
+        "interactions": Value::Array(self.interactions.iter()
+          .sorted_by(|a, b| Ord::cmp(&a.description(), &b.description()))
+          .map(|i| i.to_json()).collect()),
         "metadata": self.metadata_to_json()
       })),
       _ => if self.has_mixed_interactions() {
@@ -168,11 +233,11 @@ impl Pact for V4Pact {
     PactSpecification::V4
   }
 
-  fn boxed(&self) -> Box<dyn Pact + Send> {
+  fn boxed(&self) -> Box<dyn Pact + Send + Sync> {
     Box::new(self.clone())
   }
 
-  fn arced(&self) -> Arc<dyn Pact + Send> {
+  fn arced(&self) -> Arc<dyn Pact + Send + Sync> {
     Arc::new(self.clone())
   }
 
@@ -189,6 +254,47 @@ impl Pact for V4Pact {
       }
     }
   }
+
+  fn requires_plugins(&self) -> bool {
+    !self.plugin_data.is_empty()
+  }
+
+  fn plugin_data(&self) -> Vec<PluginData> {
+    self.plugin_data.clone()
+  }
+
+  fn is_v4(&self) -> bool {
+    true
+  }
+
+  fn add_plugin(
+    &mut self,
+    name: &str,
+    version: &str,
+    plugin_data: Option<HashMap<String, Value>>
+  ) -> anyhow::Result<()> {
+    self.add_plugin_data(&PluginData {
+      name: name.to_string(),
+      version: version.to_string(),
+      configuration: plugin_data.unwrap_or_default()
+    });
+    Ok(())
+  }
+
+  fn add_md_version(&mut self, key: &str, version: &str) {
+    if let Some(md) = self.metadata.get_mut("pactRust") {
+      match md {
+        Value::Object(map) => {
+          map.insert(key.to_string(), Value::String(version.to_string()));
+        }
+        _ => {}
+      }
+    } else {
+      self.metadata.insert("pactRust".to_string(), json!({
+        key: version
+      }));
+    }
+  }
 }
 
 impl Default for V4Pact {
@@ -197,7 +303,8 @@ impl Default for V4Pact {
       consumer: Default::default(),
       provider: Default::default(),
       interactions: vec![],
-      metadata: Default::default()
+      metadata: Default::default(),
+      plugin_data: vec![]
     }
   }
 }
@@ -208,7 +315,9 @@ impl ReadWritePact for V4Pact {
     let json = with_read_lock(path, 3, &mut |f| {
       serde_json::from_reader::<_, Value>(f).context("Failed to parse Pact JSON")
     })?;
-    let metadata = meta_data_from_json(&json);
+
+    let mut metadata = meta_data_from_json(&json);
+
     let consumer = match json.get("consumer") {
       Some(v) => Consumer::from_json(v),
       None => Consumer { name: "consumer".into() }
@@ -217,17 +326,21 @@ impl ReadWritePact for V4Pact {
       Some(v) => Provider::from_json(v),
       None => Provider { name: "provider".into() }
     };
+
+    let plugin_data = V4Pact::extract_plugin_data(&mut metadata);
+
     Ok(V4Pact {
       consumer,
       provider,
       interactions: interactions_from_json(&json, &*path.to_string_lossy()),
-      metadata
+      metadata,
+      plugin_data
     })
   }
 
-  fn merge(&self, other: &dyn Pact) -> anyhow::Result<Box<dyn Pact>> {
+  fn merge(&self, other: &dyn Pact) -> anyhow::Result<Box<dyn Pact + Send + Sync>> {
     if self.consumer.name == other.consumer().name && self.provider.name == other.provider().name {
-      Ok(Box::new(V4Pact {
+      let mut new_pact = V4Pact {
         consumer: self.consumer.clone(),
         provider: self.provider.clone(),
         interactions: self.interactions.iter()
@@ -260,8 +373,17 @@ impl ReadWritePact for V4Pact {
             }
           })
           .collect(),
-        metadata: self.metadata.clone()
-      }))
+        metadata: self.metadata.clone(),
+        plugin_data: self.plugin_data.clone()
+      };
+
+      if other.is_v4() {
+        for plugin in other.as_v4_pact().unwrap_or_default().plugin_data {
+          new_pact.add_plugin_data(&plugin);
+        }
+      }
+
+      Ok(Box::new(new_pact))
     } else {
       Err(anyhow!("Unable to merge pacts, as they have different consumers or providers"))
     }
@@ -272,6 +394,7 @@ impl ReadWritePact for V4Pact {
   }
 }
 
+
 impl PactJsonVerifier for V4Pact {
   fn verify_json(_path: &str, pact_json: &Value, _strict: bool, _spec_version: PactSpecification) -> Vec<PactFileVerificationResult> {
     let mut results = vec![];
@@ -289,8 +412,10 @@ impl PactJsonVerifier for V4Pact {
 }
 
 /// Creates a V4 Pact from the provided JSON struct
-pub fn from_json(source: &str, pact_json: &Value) -> anyhow::Result<Box<dyn Pact>> {
-  let metadata = meta_data_from_json(pact_json);
+pub fn from_json(source: &str, pact_json: &Value) -> anyhow::Result<Box<dyn Pact + Send + Sync>> {
+  trace!("from_json: Loading a V4 pact from JSON");
+  let mut metadata = meta_data_from_json(pact_json);
+
   let consumer = match pact_json.get("consumer") {
     Some(v) => Consumer::from_json(v),
     None => Consumer { name: "consumer".into() }
@@ -299,11 +424,15 @@ pub fn from_json(source: &str, pact_json: &Value) -> anyhow::Result<Box<dyn Pact
     Some(v) => Provider::from_json(v),
     None => Provider { name: "provider".into() }
   };
+
+  let plugin_data = V4Pact::extract_plugin_data(&mut metadata);
+
   Ok(Box::new(V4Pact {
     consumer,
     provider,
     interactions: interactions_from_json(pact_json, source),
-    metadata
+    metadata,
+    plugin_data
   }))
 }
 
@@ -336,11 +465,11 @@ mod tests {
   use crate::provider_states::ProviderState;
   use crate::v4::async_message::AsynchronousMessage;
   use crate::v4::http_parts::{HttpRequest, HttpResponse};
-  use crate::v4::V4InteractionType;
   use crate::v4::message_parts::MessageContents;
   use crate::v4::pact::{from_json, V4Pact};
-  use crate::v4::sync_message::SynchronousMessages;
+  use crate::v4::sync_message::SynchronousMessage;
   use crate::v4::synch_http::SynchronousHttp;
+  use crate::v4::V4InteractionType;
 
   #[test]
   fn load_empty_pact() {
@@ -387,7 +516,8 @@ mod tests {
     expect!(&pact.provider().name).to(be_equal_to("Alice Service"));
     expect!(&pact.consumer().name).to(be_equal_to("Consumer"));
     expect!(pact.interactions().iter()).to(have_count(1));
-    let interaction = pact.interactions()[0];
+    let interactions = pact.interactions();
+    let interaction = interactions.first().unwrap();
     expect!(interaction.description()).to(be_equal_to("a retrieve Mallory request"));
     expect!(interaction.provider_states().iter()).to(be_empty());
     expect!(pact.specification_version()).to(be_equal_to(PactSpecification::V4));
@@ -409,7 +539,7 @@ mod tests {
         expect!(response).to(be_equal_to(HttpResponse {
           status: 200,
           headers: Some(hashmap!{ "Content-Type".to_string() => vec!["text/html".to_string()] }),
-          body: OptionalBody::Present("\"That is some good Mallory.\"".into(), Some("text/html".into())),
+          body: OptionalBody::Present("\"That is some good Mallory.\"".into(), Some("text/html".into()), None),
           .. HttpResponse::default()
         }));
         expect!(pending).to(be_false());
@@ -543,7 +673,7 @@ mod tests {
   ],
   "metadata": {{
     "pactRust": {{
-      "version": "{}"
+      "models": "{}"
     }},
     "pactSpecification": {{
       "version": "4.0"
@@ -561,7 +691,7 @@ mod tests {
       consumer: Consumer { name: "write_pact_test_consumer".into() },
       provider: Provider { name: "write_pact_test_provider".into() },
       interactions: vec![
-        Box::new(SynchronousMessages {
+        Box::new(SynchronousMessage {
           id: None,
           key: None,
           description: "Test Interaction".into(),
@@ -590,7 +720,7 @@ mod tests {
   "interactions": [
     {{
       "description": "Test Interaction",
-      "key": "b341297869a4287d",
+      "key": "7f50f8fcda779998",
       "pending": false,
       "providerStates": [
         {{
@@ -618,7 +748,7 @@ mod tests {
   ],
   "metadata": {{
     "pactRust": {{
-      "version": "{}"
+      "models": "{}"
     }},
     "pactSpecification": {{
       "version": "4.0"
@@ -642,7 +772,8 @@ mod tests {
           .. SynchronousHttp::default()
         })
       ],
-      metadata: btreemap!{}
+      metadata: btreemap!{},
+      plugin_data: vec![]
     };
     let pact2 = V4Pact {
       consumer: Consumer { name: "merge_consumer".into() },
@@ -654,7 +785,8 @@ mod tests {
           .. SynchronousHttp::default()
         })
       ],
-      metadata: btreemap!{}
+      metadata: btreemap!{},
+      plugin_data: vec![]
     };
     let mut dir = env::temp_dir();
     let x = rand::random::<u16>();
@@ -713,7 +845,7 @@ mod tests {
   ],
   "metadata": {{
     "pactRust": {{
-      "version": "{}"
+      "models": "{}"
     }},
     "pactSpecification": {{
       "version": "4.0"
@@ -738,7 +870,8 @@ mod tests {
           .. SynchronousHttp::default()
         })
       ],
-      metadata: btreemap!{}
+      metadata: btreemap!{},
+      plugin_data: vec![]
     };
     let pact2 = V4Pact {
       consumer: Consumer { name: "write_pact_test_consumer".into() },
@@ -752,7 +885,8 @@ mod tests {
           .. SynchronousHttp::default()
         })
       ],
-      metadata: btreemap!{}
+      metadata: btreemap!{},
+      plugin_data: vec![]
     };
     let mut dir = env::temp_dir();
     let x = rand::random::<u16>();
@@ -793,7 +927,7 @@ mod tests {
   ],
   "metadata": {{
     "pactRust": {{
-      "version": "{}"
+      "models": "{}"
     }},
     "pactSpecification": {{
       "version": "4.0"
@@ -810,12 +944,14 @@ mod tests {
     let pact = V4Pact { consumer: Consumer { name: "test_consumer".to_string() },
       provider: Provider { name: "test_provider".to_string() },
       interactions: vec![],
-      metadata: btreemap!{}
+      metadata: btreemap!{},
+      plugin_data: vec![]
     };
     let pact2 = V4Pact { consumer: Consumer { name: "test_consumer2".to_string() },
       provider: Provider { name: "test_provider".to_string() },
       interactions: vec![],
-      metadata: btreemap!{}
+      metadata: btreemap!{},
+      plugin_data: vec![]
     };
     expect!(pact.merge(&pact2)).to(be_err());
   }
@@ -825,12 +961,14 @@ mod tests {
     let pact = V4Pact { consumer: Consumer { name: "test_consumer".to_string() },
       provider: Provider { name: "test_provider".to_string() },
       interactions: vec![],
-      metadata: btreemap!{}
+      metadata: btreemap!{},
+      plugin_data: vec![]
     };
     let pact2 = V4Pact { consumer: Consumer { name: "test_consumer".to_string() },
       provider: Provider { name: "test_provider2".to_string() },
       interactions: vec![],
-      metadata: btreemap!{}
+      metadata: btreemap!{},
+      plugin_data: vec![]
     };
     expect!(pact.merge(&pact2)).to(be_err());
   }
@@ -968,7 +1106,7 @@ mod tests {
   ],
   "metadata": {{
     "pactRust": {{
-      "version": "{}"
+      "models": "{}"
     }},
     "pactSpecification": {{
       "version": "2.0.0"
@@ -1065,7 +1203,7 @@ mod tests {
   ],
   "metadata": {{
     "pactRust": {{
-      "version": "{}"
+      "models": "{}"
     }},
     "pactSpecification": {{
       "version": "3.0.0"
@@ -1298,7 +1436,7 @@ mod tests {
   ],
   "metadata": {{
     "pactRust": {{
-      "version": "{}"
+      "models": "{}"
     }},
     "pactSpecification": {{
       "version": "4.0"
@@ -1327,13 +1465,13 @@ mod tests {
       .. V4Pact::default() };
     let pact4 = V4Pact {
       interactions: vec![
-        Box::new(SynchronousMessages::default())
+        Box::new(SynchronousMessage::default())
       ],
       .. V4Pact::default() };
     let pact5 = V4Pact {
       interactions: vec![
         Box::new(SynchronousHttp::default()),
-        Box::new(SynchronousMessages::default())
+        Box::new(SynchronousMessage::default())
       ],
       .. V4Pact::default() };
 
@@ -1375,13 +1513,13 @@ mod tests {
       .. V4Pact::default() };
     let pact4 = V4Pact {
       interactions: vec![
-        Box::new(SynchronousMessages::default())
+        Box::new(SynchronousMessage::default())
       ],
       .. V4Pact::default() };
     let pact5 = V4Pact {
       interactions: vec![
         Box::new(SynchronousHttp::default()),
-        Box::new(SynchronousMessages::default())
+        Box::new(SynchronousMessage::default())
       ],
       .. V4Pact::default() };
 
